%define kernel_git_commit 35ef50d2d1ca73db72d95ab9fbd1fd484f899456
Summary: The Linux kernel


# For a stable, released kernel, released_kernel should be 1. For rawhide
# and/or a kernel built from an rc or git snapshot, released_kernel should
# be 0.
%define released_kernel 1

# Versions of various parts

# Polite request for people who spin their own kernel rpms:
# please modify the "buildid" define in a way that identifies
# that the kernel isn't the stock distribution kernel, for example,
# by setting the define to ".local" or ".bz123456"
#
# % define buildid .local

%define rhel 1
%if %{rhel}
%define distro_build 100
%define signmodules 1
%else

# fedora_build defines which build revision of this kernel version we're
# building. Rather than incrementing forever, as with the prior versioning
# setup, we set fedora_cvs_origin to the current cvs revision s/1.// of the
# kernel spec when the kernel is rebased, so fedora_build automatically
# works out to the offset from the rebase, so it doesn't get too ginormous.
#
# If you're building on a branch, the RCS revision will be something like
# 1.1205.1.1.  In this case we drop the initial 1, subtract fedora_cvs_origin
# from the second number, and then append the rest of the RCS string as is.
# Don't stare at the awk too long, you'll go blind.
%define fedora_cvs_origin   1462
%define fedora_cvs_revision() %2
%global fedora_build %(echo %{fedora_cvs_origin}.%{fedora_cvs_revision $Revision: 1.1504 $} | awk -F . '{ OFS = "."; ORS = ""; print $3 - $1 ; i = 4 ; OFS = ""; while (i <= NF) { print ".", $i ; i++} }')
%define distro_build %{fedora_build}
%define signmodules 0
%endif

# base_sublevel is the kernel version we're starting with and patching
# on top of -- for example, 2.6.22-rc7-git1 starts with a 2.6.21 base,
# which yields a base_sublevel of 21.
%define base_sublevel 32

## If this is a released kernel ##
%if 0%{?released_kernel}

# Do we have a -stable update to apply?
%define stable_update 0
# Is it a -stable RC?
%define stable_rc 0
# Set rpm version accordingly
%if 0%{?stable_update}
%define stablerev .%{stable_update}
%define stable_base %{stable_update}
%if 0%{?stable_rc}
# stable RCs are incremental patches, so we need the previous stable patch
%define stable_base %(echo $((%{stable_update} - 1)))
%endif
%endif
%define rpmversion 2.6.%{base_sublevel}%{?stablerev}

## The not-released-kernel case ##
%else
# The next upstream release sublevel (base_sublevel+1)
%define upstream_sublevel %(echo $((%{base_sublevel} + 1)))
# The rc snapshot level
%define rcrev 0
# The git snapshot level
%define gitrev 0
# Set rpm version accordingly
%define rpmversion 2.6.%{upstream_sublevel}
%endif
# Nb: The above rcrev and gitrev values automagically define Patch00 and Patch01 below.

# What parts do we want to build?  We must build at least one kernel.
# These are the kernels that are built IF the architecture allows it.
# All should default to 1 (enabled) and be flipped to 0 (disabled)
# by later arch-specific checks.

# The following build options are enabled by default.
# Use either --without <opt> in your rpmbuild command or force values
# to 0 in here to disable them.
#
# standard kernel
%define with_up        1
# kernel-smp (only valid for ppc 32-bit, sparc64)
%define with_smp       1
# kernel-kdump
%define with_kdump     0
# kernel-debug
%define with_debug     1
# kernel-doc
%define with_doc       1
# kernel-headers
%define with_headers   1
# kernel-firmware
%define with_firmware  0
# kernel-debuginfo
%define with_debuginfo %{?_without_debuginfo: 0} %{?!_without_debuginfo: 1}
# kernel-bootwrapper (for creating zImages from kernel + initrd)
%define with_bootwrapper %{?_without_bootwrapper: 0} %{?!_without_bootwrapper: 1}
# Want to build a the vsdo directories installed
%define with_vdso_install %{?_without_vdso_install: 0} %{?!_without_vdso_install: 1}
# Control whether we perform a compat. check against published ABI.
%define with_kabichk 1

# Build the kernel-doc package, but don't fail the build if it botches.
# Here "true" means "continue" and "false" means "fail the build".
%if 0%{?released_kernel}
%define doc_build_fail false
%else
%define doc_build_fail true
%endif

# Additional options for user-friendly one-off kernel building:
#
# Only build the base kernel (--with baseonly):
%define with_baseonly  %{?_with_baseonly:     1} %{?!_with_baseonly:     0}
# Only build the smp kernel (--with smponly):
%define with_smponly   %{?_with_smponly:      1} %{?!_with_smponly:      0}

# should we do C=1 builds with sparse
%define with_sparse	%{?_with_sparse:      1} %{?!_with_sparse:      0}

# Set debugbuildsenabled to 1 for production (build separate debug kernels)
#  and 0 for rawhide (all kernels are debug kernels).
# See also 'make debug' and 'make release'.
%define debugbuildsenabled 1

# Want to build a vanilla kernel build without any non-upstream patches?
# (well, almost none, we need nonintconfig for build purposes). Default to 0 (off).
%define with_vanilla %{?_with_vanilla: 1} %{?!_with_vanilla: 0}

# pkg_release is what we'll fill in for the rpm Release: field
%if 0%{?released_kernel}

%if 0%{?stable_rc}
%define stable_rctag .rc%{stable_rc}
%endif
%define pkg_release %{distro_build}%{?stable_rctag}%{?dist}%{?buildid}

%else

# non-released_kernel
%if 0%{?rcrev}
%define rctag .rc%rcrev
%else
%define rctag .rc0
%endif
%if 0%{?gitrev}
%define gittag .git%gitrev
%else
%define gittag .git0
%endif
%define pkg_release 0.%{distro_build}%{?rctag}%{?gittag}%{?dist}%{?buildid}

%endif

# The kernel tarball/base version
%define kversion 2.6.%{base_sublevel}

%define make_target bzImage

%define hdrarch %_target_cpu
%define asmarch %_target_cpu

%if 0%{!?nopatches:1}
%define nopatches 0
%endif

%if %{with_vanilla}
%define nopatches 1
%endif

%if %{nopatches}
%define with_bootwrapper 0
%define variant -vanilla
%else
%define variant_fedora -fedora
%endif

%define using_upstream_branch 0
%if 0%{?upstream_branch:1}
%define stable_update 0
%define using_upstream_branch 1
%define variant -%{upstream_branch}%{?variant_fedora}
%define pkg_release 0.%{distro_build}%{upstream_branch_tag}%{?dist}%{?buildid}
%endif

%if %{rhel}
%define pkg_release %{distro_build}.35.1%{?dist}uek%{?buildid}
%endif
%define KVERREL %{rpmversion}-%{pkg_release}

%if !%{debugbuildsenabled}
%define with_debug 0
%endif

%if !%{with_debuginfo}
%define _enable_debug_packages 0
%endif
%define debuginfodir /usr/lib/debug

# We only build -PAE for 686 as of Fedora 11.
%ifarch i686
%define with_up 1
%define with_pae 0
%else
%define with_pae 0
%endif

# if requested, only build base kernel
%if %{with_baseonly}
%define with_smp 0
%define with_kdump 0
%define with_debug 0
%endif

# if requested, only build smp kernel
%if %{with_smponly}
%define with_up 0
%define with_kdump 0
%define with_debug 0
%endif

%define all_x86 i386 i686

%if %{with_vdso_install}
# These arches install vdso/ directories.
%define vdso_arches %{all_x86} x86_64 ppc ppc64
%endif

# Overrides for generic default options

# only ppc and sparc64 need separate smp kernels
%ifnarch ppc sparc64 alphaev56
%define with_smp 0
%endif

# only build kernel-kdump on ppc64
# (no relocatable kernel support upstream yet)
#FIXME: Temporarily disabled to speed up builds.
#ifnarch ppc64
%define with_kdump 0
#endif

# don't do debug builds on anything but i686 and x86_64
%ifnarch i686 x86_64
%define with_debug 0
%endif

# only package docs noarch
%ifnarch noarch
%define with_doc 0
%endif

# no need to build headers again for these arches,
# they can just use i586 and ppc64 headers
%ifarch ppc64iseries
%define with_headers 0
%endif

# don't build noarch kernels or headers (duh)
%ifarch noarch
%define with_up 0
%define with_headers 0
%define all_arch_configs kernel-%{version}-*.config
%define with_firmware  %{?_without_firmware:  0} %{?!_without_firmware:  1}
%endif

# bootwrapper is only on ppc
%ifnarch ppc ppc64
%define with_bootwrapper 0
%endif

# sparse blows up on ppc64 alpha and sparc64
%ifarch ppc64 ppc alpha sparc64
%define with_sparse 0
%endif

# Per-arch tweaks

%ifarch %{all_x86}
%define asmarch x86
%define hdrarch i386
%define all_arch_configs kernel-%{version}-i?86*.config
%define image_install_path boot
%define kernel_image arch/x86/boot/bzImage
%endif

%ifarch x86_64
%define asmarch x86
%define all_arch_configs kernel-%{version}-x86_64*.config
%define image_install_path boot
%define kernel_image arch/x86/boot/bzImage
%endif

%ifarch ppc64
%define asmarch powerpc
%define hdrarch powerpc
%define all_arch_configs kernel-%{version}-ppc64*.config
%define image_install_path boot
%define make_target vmlinux
%define kernel_image vmlinux
%define kernel_image_elf 1
%endif

%ifarch s390x
%define asmarch s390
%define hdrarch s390
%define all_arch_configs kernel-%{version}-s390x.config
%define image_install_path boot
%define make_target image
%define kernel_image arch/s390/boot/image
%endif

%ifarch sparc
# We only build sparc headers since we dont support sparc32 hardware
%endif

%ifarch sparc64
%define asmarch sparc
%define all_arch_configs kernel-%{version}-sparc64*.config
%define make_target image
%define kernel_image arch/sparc/boot/image
%define image_install_path boot
%endif

%ifarch ppc
%define asmarch powerpc
%define hdrarch powerpc
%define all_arch_configs kernel-%{version}-ppc{-,.}*config
%define image_install_path boot
%define make_target vmlinux
%define kernel_image vmlinux
%define kernel_image_elf 1
%endif

%ifarch ia64
%define all_arch_configs kernel-%{version}-ia64*.config
%define image_install_path boot/efi/EFI/redhat
%define make_target compressed
%define kernel_image vmlinux.gz
%endif

%ifarch alpha alphaev56
%define all_arch_configs kernel-%{version}-alpha*.config
%define image_install_path boot
%define make_target vmlinux
%define kernel_image vmlinux
%endif

%ifarch %{arm}
%define all_arch_configs kernel-%{version}-arm*.config
%define image_install_path boot
%define hdrarch arm
%define make_target vmlinux
%define kernel_image vmlinux
%endif

%if %{nopatches}
# XXX temporary until last vdso patches are upstream
%define vdso_arches ppc ppc64
%endif

%if %{nopatches}%{using_upstream_branch}
# Ignore unknown options in our config-* files.
# Some options go with patches we're not applying.
%define oldconfig_target loose_nonint_oldconfig
%else
%define oldconfig_target nonint_oldconfig
%endif

# To temporarily exclude an architecture from being built, add it to
# %nobuildarches. Do _NOT_ use the ExclusiveArch: line, because if we
# don't build kernel-headers then the new build system will no longer let
# us use the previous build of that package -- it'll just be completely AWOL.
# Which is a BadThing(tm).

# We don't build a kernel on i386; we only do kernel-headers there,
# and we no longer build for 31bit S390. Same for 32bit sparc and arm.
##%define nobuildarches i386 s390 sparc %{arm}
%define nobuildarches s390 sparc %{arm}

%ifarch %nobuildarches
%define with_up 0
%define with_smp 0
%define with_pae 0
%define with_kdump 0
%define with_debuginfo 0
%define _enable_debug_packages 0
%endif

%define with_pae_debug 0
%if %{with_pae}
%define with_pae_debug %{with_debug}
%endif

#
# Three sets of minimum package version requirements in the form of Conflicts:
# to versions below the minimum
#

#
# First the general kernel 2.6 required versions as per
# Documentation/Changes
#
%define kernel_dot_org_conflicts  ppp < 2.4.3-3, isdn4k-utils < 3.2-32, nfs-utils < 1.0.7-12, e2fsprogs < 1.37-4, util-linux < 2.12, jfsutils < 1.1.7-2, reiserfs-utils < 3.6.19-2, xfsprogs < 2.6.13-4, procps < 3.2.5-6.3, oprofile < 0.9.1-2

#
# Then a series of requirements that are distribution specific, either
# because we add patches for something, or the older versions have
# problems with the newer kernel or lack certain things that make
# integration in the distro harder than needed.
#
##%define package_conflicts initscripts < 7.23, udev < 063-6, iptables < 1.3.2-1, ipw2200-firmware < 2.4, iwl4965-firmware < 228.57.2, selinux-policy-targeted < 1.25.3-14, squashfs-tools < 4.0, wireless-tools < 29-3
%define package_conflicts initscripts < 7.23, udev < 063-6, iptables < 1.3.2-1, ipw2200-firmware < 2.4, selinux-policy-targeted < 1.25.3-14

#
# The ld.so.conf.d file we install uses syntax older ldconfig's don't grok.
#
%define kernel_xen_conflicts glibc < 2.3.5-1, xen < 3.0.1

# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
%define kernel_smp_obsoletes kernel-enterprise < 2.4.10
%define kernel_PAE_obsoletes kernel-smp < 2.6.17
%define kernel_PAE_provides kernel-xen = %{rpmversion}-%{pkg_release}

%ifarch x86_64
%define kernel_provides kernel%{?variant}-xen = %{rpmversion}-%{pkg_release}
%endif

# We moved the drm include files into kernel-headers, make sure there's
# a recent enough libdrm-devel on the system that doesn't have those.
%define kernel_headers_conflicts libdrm-devel < 2.4.0-0.15

#
# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
#
##%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, mkinitrd >= 6.0.61-1, kernel-firmware >= %{rpmversion}-%{pkg_release}, /sbin/new-kernel-pkg
%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, kernel-firmware >= %{rpmversion}-%{pkg_release}, /sbin/new-kernel-pkg, mkinitrd >= 5.1.19.6-61.0.2, kexec-tools >= 1.102pre-96.1.1.el5_5.2, device-mapper-multipath >= 0.4.9-23.0.1.el5, ql2xxx-firmware, oraclelinux-release

#
# This macro does requires, provides, conflicts, obsoletes for a kernel package.
#	%%kernel_reqprovconf <subpackage>
# It uses any kernel_<subpackage>_conflicts and kernel_<subpackage>_obsoletes
# macros defined above.
#
%define kernel_reqprovconf \
Provides: kernel%{?variant} = %{rpmversion}-%{pkg_release}\
Provides: kernel%{?variant}-%{_target_cpu} = %{rpmversion}-%{pkg_release}%{?1:.%{1}}\
Provides: kernel%{?variant}-drm = 4.3.0\
Provides: kernel%{?variant}-drm-nouveau = 12\
Provides: kernel%{?variant}-modeset = 1\
Provides: kernel%{?variant}-uname-r = %{KVERREL}%{?1:.%{1}}\
Provides: oracleasm = 2.0.5\
Provides: perf = %{KVERREL}%{?1:.%{1}}\
Provides: libperf.a = %{KVERREL}%{?1:.%{1}}\
Requires(pre): %{kernel_prereq}\
Conflicts: %{kernel_dot_org_conflicts}\
Conflicts: %{package_conflicts}\
%{expand:%%{?kernel%{?1:_%{1}}_conflicts:Conflicts: %%{kernel%{?1:_%{1}}_conflicts}}}\
#%{expand:%%{?kernel%{?1:_%{1}}_obsoletes:Obsoletes: %%{kernel%{?1:_%{1}}_obsoletes}}}\
%{expand:%%{?kernel%{?1:_%{1}}_provides:Provides: %%{kernel%{?1:_%{1}}_provides}}}\
# We can't let RPM do the dependencies automatic because it'll then pick up\
# a correct but undesirable perl dependency from the module headers which\
# isn't required for the kernel proper to function\
AutoReq: no\
AutoProv: yes\
%{nil}

%define variant -uek
Name: kernel%{?variant}
Group: System Environment/Kernel
License: GPLv2
URL: http://www.kernel.org/
Version: %{rpmversion}
Release: %{pkg_release}
# DO NOT CHANGE THE 'ExclusiveArch' LINE TO TEMPORARILY EXCLUDE AN ARCHITECTURE BUILD.
# SET %%nobuildarches (ABOVE) INSTEAD
ExclusiveArch: noarch %{all_x86} x86_64 ppc ppc64 ia64 sparc sparc64 s390x alpha alphaev56 %{arm}
ExclusiveOS: Linux

%kernel_reqprovconf
%ifarch x86_64
Obsoletes: kernel-smp
%endif


#
# List the packages used during the kernel build
#
BuildRequires: module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
BuildRequires: bzip2, findutils, gzip, m4, perl, make >= 3.78, diffutils, gawk
BuildRequires: gcc >= 3.4.2, binutils >= 2.12
BuildRequires: net-tools
BuildRequires: elfutils-libelf-devel
%if %{with_doc}
BuildRequires: xmlto
%endif
%if %{with_sparse}
BuildRequires: sparse >= 0.4.1
%endif
%if %{signmodules}
BuildRequires: gnupg
%endif
BuildConflicts: rhbuildsys(DiskFree) < 500Mb

Source0: ftp://ftp.kernel.org/pub/linux/kernel/v2.6/linux-%{kversion}.tar.bz2

Source11: genkey
Source14: find-provides
Source15: merge.pl
Source16: perf
Source17: kabitool
Source18: check-kabi
Source19: extrakeys.pub

Source20: Makefile.config
Source21: config-debug
Source22: config-nodebug
Source23: config-generic
Source24: config-rhel-generic
Source25: Module.kabi_i686
Source28: Module.kabi_x86_64

Source30: config-x86-generic
##Source31: config-i586
Source32: config-i686

Source40: config-x86_64-generic

Source50: config-powerpc-generic
Source51: config-powerpc32-generic
Source52: config-powerpc32-smp
Source53: config-powerpc64
##Source54: config-powerpc64-kdump

Source60: config-ia64-generic

Source70: config-s390x

Source90: config-sparc64-generic
##Source91: config-sparc64-smp

Source100: config-arm

Source101: kabi_whitelist_i686
Source102: kabi_whitelist_x86_64

# Here should be only the patches up to the upstream canonical Linus tree.

# For a stable release kernel
%if 0%{?stable_update}
%if 0%{?stable_base}
%define    stable_patch_00  patch-2.6.%{base_sublevel}.%{stable_base}.bz2
Patch00: %{stable_patch_00}
%endif
%if 0%{?stable_rc}
%define    stable_patch_01  patch-2.6.%{base_sublevel}.%{stable_update}-rc%{stable_rc}.bz2
Patch01: %{stable_patch_01}
%endif

# non-released_kernel case
# These are automagically defined by the rcrev and gitrev values set up
# near the top of this spec file.
%else
%if 0%{?rcrev}
Patch00: patch-2.6.%{upstream_sublevel}-rc%{rcrev}.bz2
%if 0%{?gitrev}
Patch01: patch-2.6.%{upstream_sublevel}-rc%{rcrev}-git%{gitrev}.bz2
%endif
%else
# pre-{base_sublevel+1}-rc1 case
%if 0%{?gitrev}
Patch00: patch-2.6.%{base_sublevel}-git%{gitrev}.bz2
%endif
%endif
%endif

%if %{using_upstream_branch}
### BRANCH PATCH ###
%endif

#Patch02: git-linus.diff

# we always need nonintconfig, even for -vanilla kernels
Patch03: nonint_oldconfig_2632rc7.patch

# we also need compile fixes for -vanilla
#Patch04: linux-2.6-compile-fixes.patch

# build tweak for build ID magic, even for -vanilla
#Patch05: linux-2.6-makefile-after_link.patch

# fix firmware install path problem (patch added/reverted after 2.6.30-rc1)
##Patch06: revert-fix-mod-install-nfs.patch

%if !%{nopatches}

# revert upstream patches we get via other methods
#Patch09: linux-2.6-upstream-reverts.patch
# Git trees.
#Patch10: git-cpufreq.patch
#Patch11: git-bluetooth.patch

# Standalone patches
#Patch20: linux-2.6-hotfixes.patch
##Patch20: linus26-blk-iopoll.patch

#Patch41: linux-2.6-sysrq-c.patch

# Fix the return code CD accesses when the CDROM drive door is closed
# but the drive isn't yet ready.
#Patch1550: linux-2.6-cdrom-door-status.patch

# fs fixes
#Patch3000: linux-2.6-btrfs-experimental-branch.patch

#Patch4000: linux-2.6-bootarg-strict-devmem.patch

# http://bugs.freedesktop.org/show_bug.cgi?id=20803
#Patch9210: linux-2.6.29-pat-fixes.patch


%endif

BuildRoot: %{_tmppath}/kernel-%{KVERREL}-root

# Override find_provides to use a script that provides "kernel(symbol) = hash".
# Pass path of the RPM temp dir containing kabideps to find-provides script.
%global _use_internal_dependency_generator 0
%define __find_provides %_sourcedir/find-provides %{_tmppath}
%define __find_requires /usr/lib/rpm/redhat/find-requires kernel


%description
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system: memory allocation, process allocation, device
input and output, etc.


%package doc
Summary: Various documentation bits found in the kernel source
Group: Documentation
%description doc
This package contains documentation files from the kernel
source. Various bits of information about the Linux kernel and the
device drivers shipped with it are documented in these files.

You'll want to install this package if you need a reference to the
options that can be passed to Linux kernel modules at load time.


%package headers
Summary: Header files for the Linux kernel for use by glibc
Group: Development/System
Obsoletes: glibc-kernheaders
Provides: glibc-kernheaders = 3.0-46
Obsoletes: kernel-headers
Provides: kernel-headers
%description headers
Kernel-headers includes the C header files that specify the interface
between the Linux kernel and userspace libraries and programs.  The
header files define structures and constants that are needed for
building most standard programs and are also needed for rebuilding the
glibc package.

%package firmware
Summary: Firmware files used by the Linux kernel
Group: Development/System
# This is... complicated.
# Look at the WHENCE file.
License: GPL+ and GPLv2+ and MIT and Redistributable, no modification permitted
%if "x%{?variant}" != "x"
Provides: kernel-firmware = %{rpmversion}-%{pkg_release}
%endif
%description firmware
Kernel-firmware includes firmware files required for some devices to
operate.

%package bootwrapper
Summary: Boot wrapper files for generating combined kernel + initrd images
Group: Development/System
Requires: gzip
%description bootwrapper
Kernel-bootwrapper contains the wrapper code which makes bootable "zImage"
files combining both kernel and initial ramdisk.

%package debuginfo-common
Summary: Kernel source files used by %{name}-debuginfo packages
Group: Development/Debug
Provides: %{name}-debuginfo-common-%{_target_cpu} = %{version}-%{release}
%description debuginfo-common
This package is required by %{name}-debuginfo subpackages.
It provides the kernel source files common to all builds.


#
# This macro creates a kernel-<subpackage>-debuginfo package.
#	%%kernel_debuginfo_package <subpackage>
#
%define kernel_debuginfo_package() \
%package %{?1:%{1}-}debuginfo\
Summary: Debug information for package %{name}%{?1:-%{1}}\
Group: Development/Debug\
Requires: %{name}-debuginfo-common-%{_target_cpu} = %{version}-%{release}\
Provides: %{name}%{?1:-%{1}}-debuginfo-%{_target_cpu} = %{version}-%{release}\
AutoReqProv: no\
%description -n %{name}%{?1:-%{1}}-debuginfo\
This package provides debug information for package %{name}%{?1:-%{1}}.\
This is required to use SystemTap with %{name}%{?1:-%{1}}-%{KVERREL}.\
%{expand:%%global debuginfo_args %{?debuginfo_args} -p '/.*/%%{KVERREL}%{?1:\.%{1}}/.*|/.*%%{KVERREL}%{?1:\.%{1}}(\.debug)?' -o debuginfo%{?1}.list}\
%{nil}

#
# This macro creates a kernel-<subpackage>-devel package.
#	%%kernel_devel_package <subpackage> <pretty-name>
#
%define kernel_devel_package() \
%package %{?1:%{1}-}devel\
Summary: Development package for building kernel modules to match the %{?2:%{2} }kernel\
Group: System Environment/Kernel\
Provides: kernel%{?variant}%{?1:-%{1}}-devel-%{_target_cpu} = %{version}-%{release}\
Provides: kernel%{?variant}-xen-devel = %{version}-%{release}%{?1:.%{1}}\
Provides: kernel%{?variant}-devel-%{_target_cpu} = %{version}-%{release}%{?1:.%{1}}\
Provides: kernel%{?variant}-devel = %{version}-%{release}%{?1:.%{1}}\
Provides: kernel%{?variant}-devel-uname-r = %{KVERREL}%{?1:.%{1}}\
AutoReqProv: no\
Requires(pre): /usr/bin/find\
%description -n kernel%{?variant}%{?1:-%{1}}-devel\
This package provides kernel headers and makefiles sufficient to build modules\
against the %{?2:%{2} }kernel package.\
%{nil}

#
# This macro creates a kernel-<subpackage> and its -devel and -debuginfo too.
#	%%define variant_summary The Linux kernel compiled for <configuration>
#	%%kernel_variant_package [-n <pretty-name>] <subpackage>
#
%define kernel_variant_package(n:) \
%package %1\
Summary: %{variant_summary}\
Group: System Environment/Kernel\
%kernel_reqprovconf\
%{expand:%%kernel_devel_package %1 %{!?-n:%1}%{?-n:%{-n*}}}\
%{expand:%%kernel_debuginfo_package %1}\
%{nil}


# First the auxiliary packages of the main kernel package.
%kernel_devel_package
%kernel_debuginfo_package


# Now, each variant package.

%define variant_summary The Linux kernel compiled for SMP machines
%kernel_variant_package -n SMP smp
%description smp
This package includes a SMP version of the Linux kernel. It is
required only on machines with two or more CPUs as well as machines with
hyperthreading technology.

Install the kernel-smp package if your machine uses two or more CPUs.


%define variant_summary The Linux kernel compiled for PAE capable machines
%kernel_variant_package PAE
%description PAE
This package includes a version of the Linux kernel with support for up to
64GB of high memory. It requires a CPU with Physical Address Extensions (PAE).
The non-PAE kernel can only address up to 4GB of memory.
Install the kernel-PAE package if your machine has more than 4GB of memory.


%define variant_summary The Linux kernel compiled with extra debugging enabled for PAE capable machines
%kernel_variant_package PAEdebug
Obsoletes: kernel-PAE-debug
%description PAEdebug
This package includes a version of the Linux kernel with support for up to
64GB of high memory. It requires a CPU with Physical Address Extensions (PAE).
The non-PAE kernel can only address up to 4GB of memory.
Install the kernel-PAE package if your machine has more than 4GB of memory.

This variant of the kernel has numerous debugging options enabled.
It should only be installed when trying to gather additional information
on kernel bugs, as some of these options impact performance noticably.


%define variant_summary The Linux kernel compiled with extra debugging enabled
%kernel_variant_package debug
%description debug
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

This variant of the kernel has numerous debugging options enabled.
It should only be installed when trying to gather additional information
on kernel bugs, as some of these options impact performance noticably.


%define variant_summary A minimal Linux kernel compiled for crash dumps
%kernel_variant_package kdump
%description kdump
This package includes a kdump version of the Linux kernel. It is
required only on machines which will use the kexec-based kernel crash dump
mechanism.


%prep
# do a few sanity-checks for --with *only builds
%if %{with_baseonly}
%if !%{with_up}%{with_pae}
echo "Cannot build --with baseonly, up build is disabled"
exit 1
%endif
%endif

%if %{with_smponly}
%if !%{with_smp}
echo "Cannot build --with smponly, smp build is disabled"
exit 1
%endif
%endif

patch_command='patch -p1 -F1 -s'
ApplyPatch()
{
  local patch=$1
  shift
  if [ ! -f $RPM_SOURCE_DIR/$patch ]; then
    exit 1;
  fi
  if ! egrep "^Patch[0-9]+: $patch\$" %{_specdir}/%{name}.spec ; then
    [ "${patch:0:10}" != "patch-2.6." ] && echo "Patch $patch not listed in specfile" && exit 1;
  fi
  case "$patch" in
  *.bz2) bunzip2 < "$RPM_SOURCE_DIR/$patch" | $patch_command ${1+"$@"} ;;
  *.gz) gunzip < "$RPM_SOURCE_DIR/$patch" | $patch_command ${1+"$@"} ;;
  *) $patch_command ${1+"$@"} < "$RPM_SOURCE_DIR/$patch" ;;
  esac
}

# First we unpack the kernel tarball.
# If this isn't the first make prep, we use links to the existing clean tarball
# which speeds things up quite a bit.

# Update to latest upstream.
%if 0%{?released_kernel}
%define vanillaversion 2.6.%{base_sublevel}
# non-released_kernel case
%else
%if 0%{?rcrev}
%define vanillaversion 2.6.%{upstream_sublevel}-rc%{rcrev}
%if 0%{?gitrev}
%define vanillaversion 2.6.%{upstream_sublevel}-rc%{rcrev}-git%{gitrev}
%endif
%else
# pre-{base_sublevel+1}-rc1 case
%if 0%{?gitrev}
%define vanillaversion 2.6.%{base_sublevel}-git%{gitrev}
%endif
%endif
%endif

# We can share hardlinked source trees by putting a list of
# directory names of the CVS checkouts that we want to share
# with in .shared-srctree. (Full pathnames are required.)
[ -f .shared-srctree ] && sharedirs=$(cat .shared-srctree)

if [ ! -d kernel-%{kversion}/vanilla-%{vanillaversion} ]; then

  if [ -d kernel-%{kversion}/vanilla-%{kversion} ]; then

    cd kernel-%{kversion}

    # Any vanilla-* directories other than the base one are stale.
    for dir in vanilla-*; do
      [ "$dir" = vanilla-%{kversion} ] || rm -rf $dir &
    done

  else

    # Ok, first time we do a make prep.
    rm -f pax_global_header
    for sharedir in $sharedirs ; do
      if [[ ! -z $sharedir  &&  -d $sharedir/kernel-%{kversion}/vanilla-%{kversion} ]] ; then
        break
      fi
    done
    if [[ ! -z $sharedir  &&  -d $sharedir/kernel-%{kversion}/vanilla-%{kversion} ]] ; then
%setup -q -n kernel-%{kversion} -c -T
      cp -rl $sharedir/kernel-%{kversion}/vanilla-%{kversion} .
    else
%setup -q -n kernel-%{kversion} -c
      mv linux-%{kversion} vanilla-%{kversion}
    fi

  fi

%if "%{kversion}" != "%{vanillaversion}"

  for sharedir in $sharedirs ; do
    if [[ ! -z $sharedir  &&  -d $sharedir/kernel-%{kversion}/vanilla-%{vanillaversion} ]] ; then
      break
    fi
  done
  if [[ ! -z $sharedir  &&  -d $sharedir/kernel-%{kversion}/vanilla-%{vanillaversion} ]] ; then

    cp -rl $sharedir/kernel-%{kversion}/vanilla-%{vanillaversion} .

  else

    cp -rl vanilla-%{kversion} vanilla-%{vanillaversion}
    cd vanilla-%{vanillaversion}

# Update vanilla to the latest upstream.
# (non-released_kernel case only)
%if 0%{?rcrev}
    ApplyPatch patch-2.6.%{upstream_sublevel}-rc%{rcrev}.bz2
%if 0%{?gitrev}
    ApplyPatch patch-2.6.%{upstream_sublevel}-rc%{rcrev}-git%{gitrev}.bz2
%endif
%else
# pre-{base_sublevel+1}-rc1 case
%if 0%{?gitrev}
    ApplyPatch patch-2.6.%{base_sublevel}-git%{gitrev}.bz2
%endif
%endif

    cd ..

  fi

%endif

else
  # We already have a vanilla dir.
  cd kernel-%{kversion}
fi

if [ -d linux-%{kversion}.%{_target_cpu} ]; then
  # Just in case we ctrl-c'd a prep already
  rm -rf deleteme.%{_target_cpu}
  # Move away the stale away, and delete in background.
  mv linux-%{kversion}.%{_target_cpu} deleteme.%{_target_cpu}
  rm -rf deleteme.%{_target_cpu} &
fi

cp -rl vanilla-%{vanillaversion} linux-%{kversion}.%{_target_cpu}

cd linux-%{kversion}.%{_target_cpu}

# released_kernel with possible stable updates
%if 0%{?stable_base}
ApplyPatch %{stable_patch_00}
%endif
%if 0%{?stable_rc}
ApplyPatch %{stable_patch_01}
%endif

%if %{using_upstream_branch}
### BRANCH APPLY ###
%endif

# Drop some necessary files from the source dir into the buildroot
cp $RPM_SOURCE_DIR/config-* .
cp %{SOURCE15} .

# Dynamically generate kernel .config files from config-* files
make -f %{SOURCE20} VERSION=%{version} configs

#if a rhel kernel, apply the rhel config options
%if 0%{?rhel}
  for i in %{all_arch_configs}
  do
    mv $i $i.tmp
    ./merge.pl config-rhel-generic $i.tmp > $i
    rm $i.tmp
  done
%endif

#ApplyPatch git-linus.diff

# This patch adds a "make nonint_oldconfig" which is non-interactive and
# also gives a list of missing options at the end. Useful for automated
# builds (as used in the buildsystem).
ApplyPatch nonint_oldconfig_2632rc7.patch

#ApplyPatch linux-2.6-makefile-after_link.patch

#
# misc small stuff to make things compile
#
##C=$(wc -l $RPM_SOURCE_DIR/linux-2.6-compile-fixes.patch | awk '{print $1}')
##if [ "$C" -gt 10 ]; then
##ApplyPatch linux-2.6-compile-fixes.patch
##fi

%if !%{nopatches}

# revert patches from upstream that conflict or that we get via other means
##C=$(wc -l $RPM_SOURCE_DIR/linux-2.6-upstream-reverts.patch | awk '{print $1}')
##if [ "$C" -gt 10 ]; then
##ApplyPatch linux-2.6-upstream-reverts.patch -R
##fi

#ApplyPatch git-cpufreq.patch
#ApplyPatch git-bluetooth.patch

#ApplyPatch linux-2.6-hotfixes.patch

# Roland's utrace ptrace replacement.
#ApplyPatch linux-2.6-tracehook.patch
#ApplyPatch linux-2.6-utrace.patch
#ApplyPatch linux-2.6-utrace-ftrace.patch

# enable sysrq-c on all kernels, not only kexec
#ApplyPatch linux-2.6-sysrq-c.patch

#ApplyPatch rds-only-on-64-bit-or-x86.patch


# Architecture patches
# x86(-64)

#
# PowerPC
#
### NOT (YET) UPSTREAM:
# The storage alias patch is Fedora-local, and allows the old 'ps3_storage'
# module name to work on upgrades. Otherwise, I believe mkinitrd will fail
# to pull the module in,
#ApplyPatch linux-2.6-ps3-storage-alias.patch
# Alleviate G5 thermal shutdown problems
#ApplyPatch linux-2.6-g5-therm-shutdown.patch
# Provide modalias in sysfs for vio devices
#ApplyPatch linux-2.6-vio-modalias.patch
# Work around PCIe bridge setup on iSight
#ApplyPatch linux-2.6-imac-transparent-bridge.patch

#
# SPARC64
#
#ApplyPatch linux-2.6.29-sparc-IOC_TYPECHECK.patch

#
# Exec shield
#
#ApplyPatch linux-2.6-execshield.patch

# Extra patches:
##ApplyPatch linus26-blk-iopoll.patch

#
# bugfixes to drivers and filesystems
#

# ext4

# xfs

# btrfs
#ApplyPatch linux-2.6-btrfs-experimental-branch.patch

# Make strict devmem a boot arg
#ApplyPatch linux-2.6-bootarg-strict-devmem.patch

# USB

# ACPI
#ApplyPatch linux-2.6-defaults-acpi-video.patch
#ApplyPatch linux-2.6-acpi-video-dos.patch
#ApplyPatch linux-2.6-hwmon-atk0110.patch

# Various low-impact patches to aid debugging.
#ApplyPatch linux-2.6-debug-sizeof-structs.patch
#ApplyPatch linux-2.6-debug-nmi-timeout.patch
#ApplyPatch linux-2.6-debug-taint-vm.patch
#ApplyPatch linux-2.6-debug-spinlock-taint.patch
#ApplyPatch linux-2.6-debug-vm-would-have-oomkilled.patch
#ApplyPatch linux-2.6-debug-always-inline-kzalloc.patch

#
# PCI
#
# disable message signaled interrupts
#ApplyPatch linux-2.6-defaults-pci_no_msi.patch
# update the pciehp driver
#ApplyPatch linux-2.6-pciehp-update.patch
# default to enabling passively listening for hotplug events
#ApplyPatch linux-2.6-defaults-pciehp.patch

#
# SCSI Bits.
#

# ALSA
# squelch hda_beep by default
#ApplyPatch linux-2.6-defaults-alsa-hda-beep-off.patch
#ApplyPatch hda_intel-prealloc-4mb-dmabuffer.patch

# Networking

# Misc fixes
# The input layer spews crap no-one cares about.
#ApplyPatch linux-2.6-input-kill-stupid-messages.patch

# Get away from having to poll Toshibas
#ApplyPatch linux-2.6-input-fix-toshiba-hotkeys.patch

# Allow to use 480600 baud on 16C950 UARTs
#ApplyPatch linux-2.6-serial-460800.patch

#ApplyPatch increase-MAX_LOCKDEP_ENTRIES.patch

# Silence some useless messages that still get printed with 'quiet'
#ApplyPatch linux-2.6-silence-noise.patch

# Make fbcon not show the penguins with 'quiet'
#ApplyPatch linux-2.6-silence-fbcon-logo.patch

# Fix the SELinux mprotect checks on executable mappings
#ApplyPatch linux-2.6-selinux-mprotect-checks.patch
# Fix SELinux for sparc
#ApplyPatch linux-2.6-sparc-selinux-mprotect-checks.patch

# Changes to upstream defaults.


# ia64 ata quirk
#ApplyPatch linux-2.6-ata-quirk.patch

# rt2x00: back-port activity LED init patches
#ApplyPatch linux-2.6-rt2x00-asus-leds.patch

# back-port scan result aging patches
#ApplyPatch linux-2.6-mac80211-age-scan-results-on-resume.patch
#ApplyPatch at76-netdev_ops.patch

# http://www.lirc.org/
#ApplyPatch linux-2.6.29-lirc.patch

# Fix the return code CD accesses when the CDROM drive door is closed
# but the drive isn't yet ready.
#ApplyPatch linux-2.6-cdrom-door-status.patch

#ApplyPatch linux-2.6-e1000-ich9.patch

# Nouveau DRM + drm fixes

# linux1394 git patches
#ApplyPatch linux-2.6-firewire-git-update.patch
C=$(wc -l $RPM_SOURCE_DIR/linux-2.6-firewire-git-pending.patch | awk '{print $1}')
#if [ "$C" -gt 10 ]; then
#ApplyPatch linux-2.6-firewire-git-pending.patch
#fi

# silence the ACPI blacklist code
#ApplyPatch linux-2.6-silence-acpi-blacklist.patch

# V4L/DVB updates/fixes/experimental drivers

# revert 8b249b6856f16f09b0e5b79ce5f4d435e439b9d6
#ApplyPatch revert-fix-modules_install-via-nfs.patch

#ApplyPatch cpufreq-add-atom-to-p4-clockmod.patch

#ApplyPatch linux-2.6.29-pat-fixes.patch

# END OF PATCH APPLICATIONS

%endif

# Any further pre-build tree manipulations happen here.

chmod +x scripts/checkpatch.pl

# only deal with configs if we are going to build for the arch
%ifnarch %nobuildarches

mkdir configs

# Remove configs not for the buildarch
for cfg in kernel-%{version}-*.config; do
  if [ `echo %{all_arch_configs} | grep -c $cfg` -eq 0 ]; then
    rm -f $cfg
  fi
done

%if !%{debugbuildsenabled}
rm -f kernel-%{version}-*debug.config
%endif

# now run oldconfig over all the config files
for i in *.config
do
  mv $i .config
  Arch=`head -1 .config | cut -b 3-`
  make ARCH=$Arch %{oldconfig_target} > /dev/null
  echo "# $Arch" > configs/$i
  cat .config >> configs/$i
done
# end of kernel config
%endif

# get rid of unwanted files resulting from patch fuzz
find . \( -name "*.orig" -o -name "*~" \) -exec rm -f {} \; >/dev/null
%if %{signmodules}
  cp %{SOURCE19} .
  gpg --homedir . --batch --gen-key %{SOURCE11}
# if there're external keys to be included
  if [ -s %{SOURCE19} ]; then
      gpg --homedir . --no-default-keyring --keyring kernel.pub --import %{SOURCE19}
  fi
  gpg --homedir . --export --keyring ./kernel.pub Oracle > extract.pub
  gcc -o scripts/bin2c scripts/bin2c.c
  scripts/bin2c ksign_def_public_key __initdata <extract.pub >crypto/signature/key.h
%endif
 	  	 

cd ..

###
### build
###
%build

%if %{with_sparse}
%define sparse_mflags	C=1
%endif

cp_vmlinux()
{
  eu-strip --remove-comment -o "$2" "$1"
}

BuildKernel() {
    MakeTarget=$1
    KernelImage=$2
    Flavour=$3
    InstallName=${4:-vmlinuz}

    # Pick the right config file for the kernel we're building
    Config=kernel-%{version}-%{_target_cpu}${Flavour:+-${Flavour}}.config
    DevelDir=/usr/src/kernels/%{KVERREL}${Flavour:+.${Flavour}}

    # When the bootable image is just the ELF kernel, strip it.
    # We already copy the unstripped file into the debuginfo package.
    if [ "$KernelImage" = vmlinux ]; then
      CopyKernel=cp_vmlinux
    else
      CopyKernel=cp
    fi

    KernelVer=%{version}-%{release}${Flavour:+.${Flavour}}
    echo BUILDING A KERNEL FOR ${Flavour} %{_target_cpu}...

    # make sure EXTRAVERSION says what we want it to say
    perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = %{?stablerev}-%{release}${Flavour:+.${Flavour}}/" Makefile

    # if pre-rc1 devel kernel, must fix up SUBLEVEL for our versioning scheme
    %if !0%{?rcrev}
    %if 0%{?gitrev}
    perl -p -i -e 's/^SUBLEVEL.*/SUBLEVEL = %{upstream_sublevel}/' Makefile
    %endif
    %endif

    # and now to start the build process

    make -s mrproper
    cp configs/$Config .config

    Arch=`head -1 .config | cut -b 3-`
    echo USING ARCH=$Arch

    make -s ARCH=$Arch %{oldconfig_target} > /dev/null
    make -s ARCH=$Arch V=1 %{?_smp_mflags} $MakeTarget %{?sparse_mflags}
    make -s ARCH=$Arch V=1 %{?_smp_mflags} modules %{?sparse_mflags} || exit 1

    # Start installing the results
%if %{with_debuginfo}
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/boot
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/%{image_install_path}
%endif
    mkdir -p $RPM_BUILD_ROOT/%{image_install_path}
    install -m 644 .config $RPM_BUILD_ROOT/boot/config-$KernelVer
    install -m 644 System.map $RPM_BUILD_ROOT/boot/System.map-$KernelVer
    touch $RPM_BUILD_ROOT/boot/initrd-$KernelVer.img
    if [ -f arch/$Arch/boot/zImage.stub ]; then
      cp arch/$Arch/boot/zImage.stub $RPM_BUILD_ROOT/%{image_install_path}/zImage.stub-$KernelVer || :
    fi
    $CopyKernel $KernelImage \
    		$RPM_BUILD_ROOT/%{image_install_path}/$InstallName-$KernelVer
    chmod 755 $RPM_BUILD_ROOT/%{image_install_path}/$InstallName-$KernelVer

    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer
    # Override $(mod-fw) because we don't want it to install any firmware
    # We'll do that ourselves with 'make firmware_install'
    make -s ARCH=$Arch INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install KERNELRELEASE=$KernelVer mod-fw=
    # check if the modules are being signed
%if %{signmodules}
      if [ -z "$(readelf -n $(find fs/ -name \*.ko | head -n 1) | grep module.sig)" ]; then
        echo "ERROR: modules are NOT signed" >&2;
	    exit 1;
      fi
%endif

	%ifarch %{vdso_arches}
    make -s ARCH=$Arch INSTALL_MOD_PATH=$RPM_BUILD_ROOT vdso_install KERNELRELEASE=$KernelVer
    if grep '^CONFIG_XEN=y$' .config >/dev/null; then
      echo > ldconfig-kernel.conf "\
# This directive teaches ldconfig to search in nosegneg subdirectories
# and cache the DSOs there with extra bit 0 set in their hwcap match
# fields.  In Xen guest kernels, the vDSO tells the dynamic linker to
# search in nosegneg subdirectories and to match this extra hwcap bit
# in the ld.so.cache file.
hwcap 0 nosegneg"
    fi
    if [ ! -s ldconfig-kernel.conf ]; then
      echo > ldconfig-kernel.conf "\
# Placeholder file, no vDSO hwcap entries used in this kernel."
    fi
    %{__install} -D -m 444 ldconfig-kernel.conf \
        $RPM_BUILD_ROOT/etc/ld.so.conf.d/kernel-$KernelVer.conf
%ifnarch noarch
# build tools/perf:
    if [ -d tools/perf ]; then
	cd tools/perf
	make all
# and install it:
	mkdir -p $RPM_BUILD_ROOT/usr/bin/$KernelVer/
	install -m 755 perf $RPM_BUILD_ROOT/usr/bin/$KernelVer/perf
	#install -m 755 libperf.a $RPM_BUILD_ROOT/lib/modules/$KernelVer/bin/%{_target_cpu}/libperf.a
	cd ../..
    fi
%endif
%endif

    # And save the headers/makefiles etc for building modules against
    #
    # This all looks scary, but the end result is supposed to be:
    # * all arch relevant include/ files
    # * all Makefile/Kconfig files
    # * all script/ files

    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/source
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    (cd $RPM_BUILD_ROOT/lib/modules/$KernelVer ; ln -s build source)
    # dirs for additional modules per module-init-tools, kbuild/modules.txt
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/extra
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/updates
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/weak-updates
    # first copy everything
    cp --parents `find  -type f -name "Makefile*" -o -name "Kconfig*"` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp Module.symvers $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp System.map $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    if [ -s Module.markers ]; then
      cp Module.markers $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    fi

    # create the kABI metadata for use in packaging
    echo "**** GENERATING kernel ABI metadata ****"
    gzip -c9 < Module.symvers > $RPM_BUILD_ROOT/boot/symvers-$KernelVer.gz
    chmod 0755 %_sourcedir/kabitool
    if [ -e $RPM_SOURCE_DIR/kabi_whitelist_%{_target_cpu}$Flavour ]; then
      cp $RPM_SOURCE_DIR/kabi_whitelist_%{_target_cpu}$Flavour $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/kabi_whitelist 
    fi 
    rm -f %{_tmppath}/kernel-$KernelVer-kabideps
    %_sourcedir/kabitool -s Module.symvers -o %{_tmppath}/kernel-$KernelVer-kabideps

%if %{with_kabichk}
    echo "**** kABI checking is enabled in kernel SPEC file. ****"
    chmod 0755 $RPM_SOURCE_DIR/check-kabi
    if [ -e $RPM_SOURCE_DIR/Module.kabi_%{_target_cpu}$Flavour ]; then
        cp $RPM_SOURCE_DIR/Module.kabi_%{_target_cpu}$Flavour $RPM_BUILD_ROOT/Module.kabi
        $RPM_SOURCE_DIR/check-kabi -k $RPM_BUILD_ROOT/Module.kabi -s Module.symvers || exit 1
        rm $RPM_BUILD_ROOT/Module.kabi # for now, don't keep it around.
    else
        echo "**** NOTE: Cannot find reference Module.kabi file. ****"
    fi
%endif

    # then drop all but the needed Makefiles/Kconfig files
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Documentation
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp .config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp -a scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    if [ -d arch/$Arch/scripts ]; then
      cp -a arch/$Arch/scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch} || :
    fi
    if [ -f arch/$Arch/*lds ]; then
      cp -a arch/$Arch/*lds $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch}/ || :
    fi
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*.o
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*/*.o
%ifarch ppc
    cp -a --parents arch/powerpc/lib/crtsavres.[So] $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/
%endif
    if [ -d arch/%{asmarch}/include ]; then
      cp -a --parents arch/%{asmarch}/include $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/
    fi
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd include
    cp -a acpi config crypto keys linux math-emu media mtd net pcmcia rdma rxrpc scsi sound trace video asm-generic xen $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    asmdir=$(readlink asm)
    cp -a $asmdir $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/
    pushd $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    ln -s $asmdir asm
    popd
    # Make sure the Makefile and version.h have a matching timestamp so that
    # external modules can be built
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Makefile $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/version.h
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    # Copy .config to include/config/auto.conf so "make prepare" is unnecessary.
    cp $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/config/auto.conf
    cd ..

    #
    # save the vmlinux file for kernel debugging into the kernel-debuginfo rpm
    #
%if %{with_debuginfo}
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/lib/modules/$KernelVer
    cp vmlinux $RPM_BUILD_ROOT%{debuginfodir}/lib/modules/$KernelVer
%endif

    find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" -type f >modnames

    # mark modules executable so that strip-to-file can strip them
    xargs --no-run-if-empty chmod u+x < modnames

    # Generate a list of modules for block and networking.

    fgrep /drivers/ modnames | xargs --no-run-if-empty nm -upA |
    sed -n 's,^.*/\([^/]*\.ko\):  *U \(.*\)$,\1 \2,p' > drivers.undef

    collect_modules_list()
    {
      sed -r -n -e "s/^([^ ]+) \\.?($2)\$/\\1/p" drivers.undef |
      LC_ALL=C sort -u > $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.$1
    }

    collect_modules_list networking \
    			 'register_netdev|ieee80211_register_hw|usbnet_probe'
    collect_modules_list block \
    			 'ata_scsi_ioctl|scsi_add_host|blk_init_queue|register_mtd_blktrans|scsi_esp_register'
    collect_modules_list drm \
    			 'drm_open|drm_init'
    collect_modules_list modesetting \
    			 'drm_crtc_init'

    # detect missing or incorrect license tags
    rm -f modinfo
    while read i
    do
      echo -n "${i#$RPM_BUILD_ROOT/lib/modules/$KernelVer/} " >> modinfo
      /sbin/modinfo -l $i >> modinfo
    done < modnames

    egrep -v \
    	  'GPL( v2)?$|Dual BSD/GPL$|Dual MPL/GPL$|GPL and additional rights$' \
	  modinfo && exit 1

    rm -f modinfo modnames

    # remove files that will be auto generated by depmod at rpm -i time
    for i in alias ccwmap dep ieee1394map inputmap isapnpmap ofmap pcimap seriomap symbols usbmap
    do
      rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.$i
    done

    # Move the devel headers out of the root file system
    mkdir -p $RPM_BUILD_ROOT/usr/src/kernels
    mv $RPM_BUILD_ROOT/lib/modules/$KernelVer/build $RPM_BUILD_ROOT/$DevelDir
    ln -sf ../../..$DevelDir $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
}

###
# DO it...
###

# prepare directories
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/boot

cd linux-%{kversion}.%{_target_cpu}

%if %{with_debug}
%if %{with_up}
BuildKernel %make_target %kernel_image debug
%endif
%if %{with_pae}
BuildKernel %make_target %kernel_image PAEdebug
%endif
%endif

%if %{with_pae}
BuildKernel %make_target %kernel_image PAE
%endif

%if %{with_up}
BuildKernel %make_target %kernel_image
%endif

%if %{with_smp}
BuildKernel %make_target %kernel_image smp
%endif

%if %{with_kdump}
BuildKernel vmlinux vmlinux kdump vmlinux
%endif

%if %{with_doc}
# Make the HTML and man pages.
make -j1  htmldocs mandocs || %{doc_build_fail}

# sometimes non-world-readable files sneak into the kernel source tree
chmod -R a=rX Documentation
find Documentation -type d | xargs chmod u+w
%endif

###
### Special hacks for debuginfo subpackages.
###

# This macro is used by %%install, so we must redefine it before that.
%define debug_package %{nil}

%if %{with_debuginfo}
%ifnarch noarch
%global __debug_package 1
%files debuginfo-common
%defattr(-,root,root)
/usr/src/debug/kernel-%{version}/linux-%{kversion}.%{_target_cpu}
%dir /usr/src/debug
%dir %{debuginfodir}
%dir %{debuginfodir}/%{image_install_path}
%dir %{debuginfodir}/lib
%dir %{debuginfodir}/lib/modules
%dir %{debuginfodir}/usr/src/kernels
%endif
%endif

###
### install
###

%install

cd linux-%{kversion}.%{_target_cpu}

%if %{with_doc}
docdir=$RPM_BUILD_ROOT%{_datadir}/doc/kernel-doc-%{rpmversion}
man9dir=$RPM_BUILD_ROOT%{_datadir}/man/man9

# copy the source over
mkdir -p $docdir
tar -f - --exclude=man --exclude='.*' -c Documentation | tar xf - -C $docdir

# Install man pages for the kernel API.
mkdir -p $man9dir
find Documentation/DocBook/man -name '*.9.gz' -print0 |
xargs -0 --no-run-if-empty %{__install} -m 444 -t $man9dir $m
ls $man9dir | grep -q '' || > $man9dir/BROKEN
%endif

%ifnarch noarch
# perf shell wrapper
mkdir -p $RPM_BUILD_ROOT/usr/sbin/
cp $RPM_SOURCE_DIR/perf $RPM_BUILD_ROOT/usr/sbin/perf
chmod 0755 $RPM_BUILD_ROOT/usr/sbin/perf
%endif

%if %{with_headers}
# Install kernel headers
make ARCH=%{hdrarch} INSTALL_HDR_PATH=$RPM_BUILD_ROOT/usr headers_install

# Do headers_check but don't die if it fails.
make ARCH=%{hdrarch} INSTALL_HDR_PATH=$RPM_BUILD_ROOT/usr headers_check \
     > hdrwarnings.txt || :
if grep -q exist hdrwarnings.txt; then
   sed s:^$RPM_BUILD_ROOT/usr/include/:: hdrwarnings.txt
   # Temporarily cause a build failure if header inconsistencies.
   # exit 1
fi

find $RPM_BUILD_ROOT/usr/include \
     \( -name .install -o -name .check -o \
     	-name ..install.cmd -o -name ..check.cmd \) | xargs rm -f

# glibc provides scsi headers for itself, for now
rm -rf $RPM_BUILD_ROOT/usr/include/scsi
rm -f $RPM_BUILD_ROOT/usr/include/asm*/atomic.h
rm -f $RPM_BUILD_ROOT/usr/include/asm*/io.h
rm -f $RPM_BUILD_ROOT/usr/include/asm*/irq.h

# these are provided by drm-devel
rm -rf $RPM_BUILD_ROOT/usr/include/drm
%endif

%if %{with_firmware}
mkdir -p $RPM_BUILD_ROOT/lib/firmware
make INSTALL_FW_PATH=$RPM_BUILD_ROOT/lib/firmware firmware_install
%endif

%if %{with_bootwrapper}
make DESTDIR=$RPM_BUILD_ROOT bootwrapper_install WRAPPER_OBJDIR=%{_libdir}/kernel-wrapper WRAPPER_DTSDIR=%{_libdir}/kernel-wrapper/dts
%endif

###
### clean
###

%clean
rm -rf $RPM_BUILD_ROOT

###
### scripts
###

#
# This macro defines a %%post script for a kernel*-devel package.
#	%%kernel_devel_post [<subpackage>]
#
%define kernel_devel_post() \
%{expand:%%post %{?1:%{1}-}devel}\
if [ -f /etc/sysconfig/kernel ]\
then\
    . /etc/sysconfig/kernel || exit $?\
fi\
if [ "$HARDLINK" != "no" -a -x /usr/sbin/hardlink ]\
then\
    (cd /usr/src/kernels/%{KVERREL}%{?1:.%{1}} &&\
     /usr/bin/find . -type f | while read f; do\
       hardlink -c /usr/src/kernels/*.fc*.*/$f $f\
     done)\
fi\
%{nil}

# This macro defines a %%posttrans script for a kernel package.
#	%%kernel_variant_posttrans [<subpackage>]
# More text can follow to go at the end of this variant's %%post.
#
###%define kernel_variant_posttrans() \
###%{expand:%%posttrans %{?1}}\
###/sbin/new-kernel-pkg --package kernel%{?1:-%{1}} --rpmposttrans %{KVERREL}%{?1:.%{1}} || exit $?\
###%{nil}

#
# This macro defines a %%post script for a kernel package and its devel package.
#	%%kernel_variant_post [-v <subpackage>] [-r <replace>]
# More text can follow to go at the end of this variant's %%post.
#
%define kernel_variant_post(v:r:) \
%{expand:%%kernel_devel_post %{?-v*}}\
%{expand:%%kernel_variant_posttrans %{?-v*}}\
%{expand:%%post %{?-v*}}\
%{-r:\
if [ `uname -i` == "x86_64" -o `uname -i` == "i386" ] &&\
   [ -f /etc/sysconfig/kernel ]; then\
  /bin/sed -r -i -e 's/^DEFAULTKERNEL=%{-r*}$/DEFAULTKERNEL=kernel%{?-v:-%{-v*}}/' /etc/sysconfig/kernel || exit $?\
fi}\
/sbin/new-kernel-pkg --package kernel%{?-v:-%{-v*}} --mkinitrd --depmod --install %{KVERREL}%{?-v:.%{-v*}} || exit $?\
if [ -x /sbin/weak-modules ]\
then\
    /sbin/weak-modules --add-kernel %{KVERREL}%{?-v:.%{-v*}} || exit $?\
fi\
%{nil}

#
# This macro defines a %%preun script for a kernel package.
#	%%kernel_variant_preun <subpackage>
#
%define kernel_variant_preun() \
%{expand:%%preun %{?1}}\
/sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}%{?1:.%{1}} || exit $?\
if [ -x /sbin/weak-modules ]\
then\
    /sbin/weak-modules --remove-kernel %{KVERREL}%{?1:.%{1}} || exit $?\
fi\
%{nil}

#
# This macro defines a %%pre script for a kernel package.
#	%%kernel_variant_pre <subpackage>
#
%define kernel_variant_pre() \
%{expand:%%pre %{?1}}\
message="Change references of /dev/hd in /etc/fstab to disk label"\
if [ -f /etc/fstab ]\
then\
awk '($2=="/boot")&&/^\\/dev\\/hd/{print $1}' /etc/fstab | egrep -q "^/dev/hd"\
bdretval=$?\
awk '($2=="/")&&/^\\/dev\\/hd/{print $1}' /etc/fstab | egrep -q "^/dev/hd"\
rdretval=$?\
awk '($2=="/boot")&&/^LABEL=/{print $1}' /etc/fstab | egrep -q "^LABEL="\
blretval=$?\
awk '($2=="/")&&/^LABEL=/{print $1}' /etc/fstab | egrep -q "^LABEL="\
rlretval=$?\
if [ $bdretval == 0 ] || [ $rdretval == 0 ]\
then\
echo -e $message\
exit 1\
elif [ $blretval == 0 ] && [ $rlretval == 0 ]\
then\
grep -v "^#" /etc/fstab | egrep -q "/dev/hd"\
if [ $? == 0 ]\
then\
echo -e $message\
fi\
elif [ $blretval == 0 ] && [ $rdretval != 0 ]\
then\
grep -v "^#" /etc/fstab | egrep -q "/dev/hd"\
if [ $? == 0 ]\
then\
echo -e $message\
fi\
elif [ $bdretval != 0 ] && [ $rlretval == 0 ]\
then\
grep -v "^#" /etc/fstab | egrep -q "/dev/hd"\
if [ $? == 0 ]\
then\
echo -e $message\
fi\
elif [ $bdretval != 0 ] && [ $rdretval != 0 ]\
then\
grep -v "^#" /etc/fstab | egrep -q "/dev/hd"\
if [ $? == 0 ]\
then\
echo -e $message\
fi\
fi\
fi\
%{nil}

%kernel_variant_pre
%kernel_variant_preun
%ifarch x86_64
%kernel_variant_post -r (kernel-smp|kernel-xen)
%else
%kernel_variant_post -r kernel-smp
%endif

%kernel_variant_pre smp
%kernel_variant_preun smp
%kernel_variant_post -v smp

%kernel_variant_pre PAE
%kernel_variant_preun PAE
%kernel_variant_post -v PAE -r (kernel|kernel-smp|kernel-xen)

%kernel_variant_pre debug
%kernel_variant_preun debug
%kernel_variant_post -v debug

%kernel_variant_post -v PAEdebug -r (kernel|kernel-smp|kernel-xen)
%kernel_variant_preun PAEdebug
%kernel_variant_pre PAEdebug

if [ -x /sbin/ldconfig ]
then
    /sbin/ldconfig -X || exit $?
fi

###
### file lists
###

%if %{with_headers}
%files headers
%defattr(-,root,root)
/usr/include/*
%endif

%if %{with_firmware}
%files firmware
%defattr(-,root,root)
/lib/firmware/*
%doc linux-%{kversion}.%{_target_cpu}/firmware/WHENCE
%endif

%if %{with_bootwrapper}
%files bootwrapper
%defattr(-,root,root)
/usr/sbin/*
%{_libdir}/kernel-wrapper
%endif

# only some architecture builds need kernel-doc
%if %{with_doc}
%files doc
%defattr(-,root,root)
%{_datadir}/doc/kernel-doc-%{rpmversion}/Documentation/*
%dir %{_datadir}/doc/kernel-doc-%{rpmversion}/Documentation
%dir %{_datadir}/doc/kernel-doc-%{rpmversion}
%{_datadir}/man/man9/*
%endif

# This is %{image_install_path} on an arch where that includes ELF files,
# or empty otherwise.
%define elf_image_install_path %{?kernel_image_elf:%{image_install_path}}

#
# This macro defines the %%files sections for a kernel package
# and its devel and debuginfo packages.
#	%%kernel_variant_files [-k vmlinux] <condition> <subpackage>
#
%define kernel_variant_files(k:) \
%if %{1}\
%{expand:%%files %{?2}}\
%defattr(-,root,root)\
/%{image_install_path}/%{?-k:%{-k*}}%{!?-k:vmlinuz}-%{KVERREL}%{?2:.%{2}}\
/boot/System.map-%{KVERREL}%{?2:.%{2}}\
/boot/symvers-%{KVERREL}%{?2:.%{2}}.gz\
/boot/config-%{KVERREL}%{?2:.%{2}}\
%dir /lib/modules/%{KVERREL}%{?2:.%{2}}\
/lib/modules/%{KVERREL}%{?2:.%{2}}/kernel\
/lib/modules/%{KVERREL}%{?2:.%{2}}/build\
/lib/modules/%{KVERREL}%{?2:.%{2}}/source\
/lib/modules/%{KVERREL}%{?2:.%{2}}/extra\
/lib/modules/%{KVERREL}%{?2:.%{2}}/updates\
/lib/modules/%{KVERREL}%{?2:.%{2}}/weak-updates\
%ifarch %{vdso_arches}\
/lib/modules/%{KVERREL}%{?2:.%{2}}/vdso\
/etc/ld.so.conf.d/kernel-%{KVERREL}%{?2:.%{2}}.conf\
%endif\
/lib/modules/%{KVERREL}%{?2:.%{2}}/modules.*\
/usr/bin/%{KVERREL}%{?2:.%{2}}/perf\
/usr/sbin/perf\
%ghost /boot/initrd-%{KVERREL}%{?2:.%{2}}.img\
%{expand:%%files %{?2:%{2}-}devel}\
%defattr(-,root,root)\
%dir /usr/src/kernels\
%verify(not mtime) /usr/src/kernels/%{KVERREL}%{?2:.%{2}}\
/usr/src/kernels/%{KVERREL}%{?2:.%{2}}\
%if %{with_debuginfo}\
%ifnarch noarch\
%{expand:%%files %{?2:%{2}-}debuginfo}\
%defattr(-,root,root)\
%{debuginfodir}/lib/modules/%{KVERREL}%{?2:.%{2}}\
%{debuginfodir}/usr/src/kernels/%{KVERREL}%{?2:.%{2}}\
%{debuginfodir}/usr/bin/%{KVERREL}%{?2:.%{2}}\
%endif\
%endif\
%endif\
%{nil}


%kernel_variant_files %{with_up}
%kernel_variant_files %{with_smp} smp
%if %{with_up}
%kernel_variant_files %{with_debug} debug
%endif
%kernel_variant_files %{with_pae} PAE
%kernel_variant_files %{with_pae_debug} PAEdebug
%kernel_variant_files -k vmlinux %{with_kdump} kdump

%changelog
* Thu Jun 2 2011 Joe Jin [2.6.32-100.35.1.el5uek]
- [net] dccp: handle invalid feature options length {CVE-2011-1770}
- [net] can: add missing socket check in can/raw release {CVE-2011-1748}
- [net] can: Add missing socket check in can/bcm release {CVE-2011-1598}

* Wed May 25 2011 Guru Anbalagane [2.6.32-100.34.1.el5uek]
- remerge netconsole over bonding support again [orabug 12591109]

* Tue May 24 2011 Guru Anbalagane [2.6.32-100.33.1.el5uek]
- add kabi whitelist [orabug 12578609]

* Tue May 24 2011 Joe Jin [2.6.32-100.32.1.el5uek]
- [x86] mce: Add HW_ERR printk prefix for hardware error logging
- [86] mce: Drop the default decoding notifier
- [86] MCE: Do not taint when handling correctable errors
- [mce] edac: Use an atomic notifier for MCEs decoding
- [netdrv] mlx4: merged some fixes for mlx4
- [kernel]: add pr_*_once, pr_*_ratelimited family

* Mon May 23 2011 Joe Jin [2.6.32-100.31.3.el5uek]
- [net] Extend prot->slab size when add sock extend fields.

* Mon May 23 2011 Joe Jin <joe.jin@oracle.com>  [2.6.32-100.31.2.el5uek]
- [virt] xen-blkfront: add missing define for vdisk.
- [virt] xen-blkfront: merge some fixes.
- [firmware] bnx2x: remove useless firmware files
- ALSA: caiaq - Fix possible string-buffer overflow
- [input] wacom: update Wacom tablet driver.
- [SCSI] mvsas: update to version 0.8.2.
- [SCSI] fnic: update fcoe_libfc_config() to reflect fcoe update.
- [SCSI] bnx2fc: updata fcoe_ctlr_recv_flogi to reflect fcoe update.
- [SCSI] megaraid_sas: updata driver to version 00.00.05.34-rc1
- [SCSI] mptsas: upgrade version to 08.101.00.00
- [SCSI] isci: merge fixes for isci
- x86: Introduce pci_map_biosrom()
- [SCSI] lpfc: update version to 8.3.5.30.1p.
- [SCSI] ipr: Fix a race on multiple configuration changes
- [SCSI] libsas: merged some fixes
- [SCSI] fcoe: merge some fixes for fcoe
- lib/random32: export pseudo-random number generator for modules
- [SCSI] libfc: merge some fixes from rhel6u1
- [SCSI] hpsa: upgrade to version 2.0.2-3
- [netdrv] bonding: update version to 3.6.0.
- [netdrv] sfc: update to version 3.0
- ethtool: Add ethtool_reset flags
- net: Fix definition of netif_vdbg() when VERBOSE_DEBUG is not defined
- qlcnic: limit skb frags for non tso packet
- [netdrv] enic: update to version 2.1.1.13
- [SCSI] bfa: add tech preview info for bfad.
- [netdrv] bna: Avoid kernel panic in case of FW heartbeat failure
- [netdrv] igb: upgrade igb to 3.0.6-k2
- [netdrv] netxen: limit skb frags for non tso packet
- [SCSI] be2scsi: Upgrade be2iscsi to 2.103.298.0
- [netdrv] be2net: merge RHEL6U1 be2net fixes.
- [netdrv] niu: Fix races between up/down and get_stats.
- [netdrv] update bnx2, bnx2x firmware
- [SCSI] bnx2fc: upgrade driver ot 1.0.2
- [netdrv] cnic: upgrade driver to 2.2.14
- [netdrv] bnx2: upgrade driver to 2.1.6.
- [netdrv] bnx2x: upgrade to 1.62.00-6
- [INFINIBAND] RDMA/cxgb4: driver upgrade!
- [netdrv] Adding Chelsio Firmware for cxgb4.
- [SCSI] export fc_exch_mgr_list_clone.
- dcbnl: adding DCBX feature flags get-set
- dcbnl: add appliction tlv handlers
- dcbnl: adding DCBX engine capability
- dcbnl: add __GENKSYMS__ trick for ieee8021Qaz ops
- dcbnl: add support for ieee8021Qaz attributes
- remove DCB_PROTO_VERSION as we don't do netlink versioning
- kernel: Fix unlimited socket backlog DoS {CVE-2010-4251}
- RDS: Fix congestion issues for loopback
- rds: prevent BUG_ON triggering on congestion map updates {CVE-2011-1023}
- epoll: prevent creating circular epoll structures {CVE-2011-1082}
- fs: fix corrupted OSF partition table parsing {CVE-2011-1163}
- fs: Increase OSF partition limit from 8 to 18 {CVE-2011-1163}
- netfilter: arp_tables: fix infoleak to userspace {CVE-2011-1170}
- netfilter: ip_tables: fix infoleak to userspace {CVE-2011-1171}
- ipv6: netfilter: ip6_tables: fix infoleak to userspace {CVE-2011-1172}
- [SCSI] mpt2sas: prevent heap overflows and unchecked reads {CVE-2011-1494, CVE-2011-1495}
- sctp: fix to calc the INIT/INIT-ACK chunk length correctly is set {CVE-2011-1573}
- dccp: fix oops on Reset after close {CVE-2011-1093}
- bridge: netfilter: fix information leak {CVE-2011-1080}
- Bluetooth: bnep: fix buffer overflow {CVE-2011-1079}
- net: don't allow CAP_NET_ADMIN to load non-netdev kernel modules {CVE-2011-1019}
- ipip: add module alias for tunl0 tunnel device
- gre: add module alias for gre0 tunnel device
- drm/radeon/kms: check AA resolve registers on r300 {CVE-2011-1016}
- drm/radeon: fix regression with AA resolve checking {CVE-2011-1016}
- drm: fix unsigned vs signed comparison issue in modeset ctl ioctl {CVE-2011-1013}
- proc: protect mm start_code/end_code in /proc/pid/stat {CVE-2011-0726}
- ALSA: caiaq - Fix possible string-buffer overflow {CVE-2011-0712}
- xfs: zero proper structure size for geometry calls {CVE-2011-0711}
- xfs: prevent leaking uninitialized stack memory in FSGEOMETRY_V1 {CVE-2011-0711}
- ima: fix add LSM rule bug {CVE-2011-0006}
- IB/uverbs: Handle large number of entries in poll CQ {CVE-2010-4649, CVE-2011-1044}
- CAN: Use inode instead of kernel address for /proc file {CVE-2010-4565}
- IB/qib: fix qib compile warning.
- IB/core: Allow device-specific per-port sysfs files.
- dm crypt: add plain64 iv.
- firmware: add firmware for qib.
- Infiniband: Add QLogic PCIe QLE InfiniBand host channel adapters support.

* Tue May 10 2011 Guru Anbalagane <guru.anbalagane@oracle.com> [2.6.32-100.31.1.el5uek]
- Update pvhvm patch 
- fix config changes for 32bit [orabug 12423905]
- version back to 100 series

* Fri Apr 29 2011 Joe Jin <joe.jin@oracle.com> [2.6.32-200.0.4.el5]
- IB/core: Allow device-specific per-port sysfs files
- IB/qib: fix qib compile warning.
- RDMA/cxgb4: Add driver for Chelsio T4 RNIC.
- net: Print num_rx_queues imbalance warning only when there are allocated queues

* Thu Apr 28 2011 Guru Anbalagane <guru.anbalagane@oracle.com> [2.6.32-200.0.3.el5]
- isci: update firmware from upstream

* Wed Apr 27 2011 Joe Jin <joe.jin@oracle.com> [2.6.32-200.0.2.el5]
- [SCSI] scsi_dh_rdac: Add TP9700 into device list
- [SCSI] scsi_dh_rdac: Add ArrayStorage into device list
- [SCSI] cciss: Add unresettable table for kdump.
- [SCSI] hpsa: Add unresettable table for kdump.
- [scsi] isci: move firmware file 

- [USB] usb: build ehci_hcd into kernel

* Fri Apr 22 2011 Guru Anbalagane <guru.anbalagane@oracle.com> [2.6.32-200.0.1.el5]
- scsi: Add some new functions for libfc and fcoe from upstream.
- scsi: adjust call parameter in fnic for new libfc APIs.
- [SCSI] bnx2fc: Broadcom FCoE offload driver
- dm mpath: hold io until all pg_inits completed
- dm mpath: disable blk_abort_queue
- dm mpath: fail message ioctl if specified path is not valid
- dm mpath: allow table load with no priority groups
- Merge pvhvm backport to UEK 
- x86, mm: avoid possible bogus tlb entries by clearing prev mm_cpumask after sw
- ixgbe: fix for 82599 erratum on Header Splitting
- dm crypt: add plain64 iv [orabug 11697886]

* Tue Apr 19 2011 Guru Anbalagane [2.6.32-100.28.13.el5]
- pci: add pci_vpd_* helper.
- net-core: add some functions that used by netdev driver.
- lib: add some basic functions in lib.
- kernel: add pr_warn define.
- raid: add RAID_LEVEL_1E define
- IB/core: Add support for masked atomic operations
- netdrv: upgrade tg3 to 3.113.
- netdrv: upgrade bnx2 driver and firmware to 2.0.17
- netdrv: upgrade bnx2x to 1.62.00-2
- netdrv: upgrade cnic to 2.2.13
- scsi: upgrade bnx2i to 2.6.2.3
- netdrv: upgrade be2net to 2.103.298r
- scsi: upgrade be2iscsi to 2.103.298r
- netdrv: upgrade e100 to 3.5.24-k2
- netdrv: upgrade e1000 to 7.3.21-k6-NAPI
- netdrv: upgrade e1000e to 1.2.20
- netdrv: upgrade cxgb3 to 1.1.4-ko
- netdrv: add Chelsio T4(cxgb4) support
- netdrv: upgrade bna to 2.3.2.3r.
- scsi: upgrade bfa to 2.3.2.3
- netdrv: upgrade igbvf to 1.0.8-k0
- netdrv: upgrade ixgbevf to 1.0.19-k0
- netdrv: remove unused kcompat_ethtool.c for igb, igbe
- netdrv: upgrade qlge to v1.00.00.27.00.00-01
- netdrv: upgrade vxge to 2.0.9-20840-k
- scsi: upgrade cxgb3i to 2.0.0
- scsi: enable cxgb4i support and upgrade it to 0.9.1
- scsi: upgrade lpfc to 0:8.3.5.30
- scsi: upgrade megaraid_sas to 00.00.05.29-RH1
- scsi: add new files for megaraid_sas driver.
- scsi: upgrade mpt2sas to 08.101.00.00
- message: upgrade mptsas to 3.04.18
- scsi: upgrade ipr to 2.5.1
- scsi: upgrade fnic to 1.5.0.1
- scsi: upgrade hpsa to 2.0.2-3
- scsi: upgrade cciss to 3.6.28-RH
- infiniband: upgrade nes to 1.5.0.0
- infiniband: upgrade mlx4 to the el6u1
- firmware: add firmware for qib
- scsi: add new devics to alua support list.
- scsi: add some devs to RDAC support list
- libsas: fix bug for vacant phy
- libsas: fix definition of wideport, include local sas address
- libsas: Don't issue commands to devices that have been hot-removed
- isci: Add support for isci driver from upstream

* Tue Apr 14 2011 Joe Jin [2.6.32-100.28.12.el5]
- NET: upgrade ixgbe to 3.2.10.
- NET: upgrade igb to 2.4.13.
- NET: upgrade qlcnic to 5.0.15.2
- NET: upgrade netxen_nic to 4.0.75
- SCSI: upgrade qla4xxx to v5.02.12.00.32.01-c0
- SCSI: upgrade qla2xxx driver to v8.03.07.03.32.1-k.
- IB: add QLogic PCIe QLE InfiniBand host channel adapters support.

* Wed Apr 13 2011 Joe Jin [2.6.32-100.28.11.el5]
- fs/partitions: Validate map_count in Mac partition tables {CVE-2011-1010}
- nfs4: Ensure that ACL pages sent over NFS were not allocated from 
  the slab (v3) {CVE-2011-1090}

* Wed Apr 13 2011 Joe Jin [2.6.32-100.28.10.el5]
- Use cciss for some Smart Array controller for OL5 [orabug 11899706]
- Add CONFIG_UEK5 option [orabug 11899706]
- CVEs from RHSA-2011-0421
- install_special_mapping skips security_file_mmap check {CVE-2010-4346}
- orinoco: fix TKIP countermeasure behaviour {CVE-2010-4648}
- net: clear heap allocation for ethtool_get_regs() {CVE-2010-4655}
- usb: iowarrior: don't trust report_size for buffer size {CVE-2010-4656}
- [media] [v3,media] av7110: check for negative array offset {CVE-2011-0521}
- RDMA/cma: Fix crash in request handlers {CVE-2011-0695}
- IB/cm: Bump reference count on cm_id before invoking callback {CVE-2011-0695}
- gro: reset skb_iif on reuse {CVE-2011-1478}

* Wed Mar 16 2011 Guru Anbalagane [2.6.32-100.28.9.el5]
- sync up with uek6 version 

* Tue Mar 15 2011 Chuck Anderson [2.6.32-100.26.4.el5]
- [block] check for proper length of iov entries earlier in blk_rq_map_user_iov
  (Xiaotian Feng) {CVE-2010-4668}
- scm: lower SCM_MAX_FD (Eric Dumazet) {CVE-2010-4249}
- perf_events: Fix perf_counter_mmap() hook in mprotect() (Pekka Enberg)
  {CVE-2010-4169}
- tcp: Increase TCP_MAXSEG socket option minimum (David S. Miller)
  {CVE-2010-4165}
- Enable module force load option [orabug 11782146]
- Enable vmw balloon and pvscsi (Guru Anbalagane) [orabug 11697522]
- fix hpilo module option in config

* Wed Mar  2 2011 Guru Anbalagane [2.6.32-100.26.3.el5]
- build from git 

* Tue Jan 18 2011 Kevin Lyons [2.6.32-100.26.2.el5]
- Make sure call to weak-modules in post and preun sections works
  for debug package [bug 11655695]

* Thu Jan 14 2011 Srinivas Maturi [2.6.32-100.26.1.el5]
- Rebuild changing release to 100.26.1.el5

* Thu Jan 14 2011 Srinivas Maturi [2.6.32-100.23.34.el5]
- Add xenbus symbols to kabi whitelist and update Module.kabi_<arch><flavor>

* Thu Jan 13 2011 Guru Anbalagane [2.6.32-100.23.33.el5]
- setup_arg_pages: diagnose excessive argument size (Roland McGrath) {CVE-2010-3858}
- [net] Truncate recvfrom and sendto length to INT_MAX (Linux Torvalds) {CVE-2010-3859}
- [net] packet: fix information leak to userland (Vasiliy Kulikov) {CVE-2010-3876}
- [net] inet_diag: Make sure we run the same bytecode we audited (Nelson Elhage) {CVE-2010-3880}
- [ipc] sys_semctl: fix kernel stack leakage (Dan Rosenberg)  {CVE-2010-4083}
- [scsi] gdth: integer overflow in ioctl (Dan Carpenter) {CVE-2010-4157}
- bluetooth: Fix missing NULL check (Alan Cox) {CVE-2010-4242}
- posix-cpu-timers: workaround to suppress the problems with mt exec (Oleg Nesterov) {CVE-2010-4248}
- [fs] xfs: prevent reading uninitialized stack memory (Dan Rosenberg) {CVE-2010-3078}
- ALSA: seq/oss - Fix double-free at error path of snd_seq_oss_open() (Takashi Iwai) {CVE-2010-3080}
- drivers/net/usb/hso.c: prevent reading uninitialized memory (Dan Rosenberg) {CVE-2010-3298}
- net: clear heap allocation for ETHTOOL_GRXCLSRLALL (Kees Cook) {CVE-2010-3861)
- can-bcm: fix minor heap overflow (Oliver Hartkopp) {CVE-2010-3874}
- ipc: shm: fix information leak to userland (Vasiliy Kulikov) {CVE-2010-4072}
- ipc: initialize structure memory to zero for compat functions (Dan Rosenberg) {CVE-2010-4073}
- USB: serial/mos*: prevent reading uninitialized stack memory (Dan Rosenberg) {CVE-2010-4074}
- tty: Make tiocgicount a handler (Alan Cox) {CVE-2010-4075}
- tty: icount changeover for other main devices (Alan Cox) {CVE-2010-4077}
- V4L/DVB: ivtvfb: prevent reading uninitialized stack memory (Dan Rosenberg) {CVE-2010-4079}
- sound/pci/rme9652: prevent reading uninitialized stack memory (Dan Rosenberg) {CVE-2010-4080}
- drivers/video/via/ioctl.c: prevent reading uninitialized stack memory (Dan Rosenberg) {CVE-2010-4082}
- filter: make sure filters dont read uninitialized memory (David S. Miller) {CVE-2010-4158}
- net: Limit socket I/O iovec total length to INT_MAX.(David S. Miller) {CVE-2010-4160}
- bio: take care not overflow page count when mapping/copying user data (Jens Axboe) {CVE-2010-4162}
- block: check for proper length of iov entries in blk_rq_map_user_iov() (Jens Axboe) {CVE-2010-4163}
- af_unix: limit unix_tot_inflight (Eric Dumazet) {CVE-2010-4249}
- igb: only use vlan_gro_receive if vlans are registered (Alexander Duyck) {CVE-2010-4263}
- net: tipc: fix information leak to userland (Kulikov Vasiliy) {CVE-2010-3877}
- exec: make argv/envp memory visible to oom-killer (Oleg Nesterov) {CVE-2010-4243}
- exec: copy-and-paste the fixes into compat_do_execve() paths (Oleg Nesterov) {CVE-2010-4243}
- oracleasm: Data integrity support (Martin K. Petersen)
- sd: Export effective protection mode in sysfs (Martin K. Petersen)
- block: Disable autoprotect (Martin K. Petersen)
- block: Fix double free (Martin K. Petersen)
- scsi_transport_fc: Introduce helper function for blocking scsi_eh (Christof Schmitt)
- scsi_transport_fc: Remove capping from dev_loss_tmo (Hannes Reinecke)

* Thu Jan 13 2011 Guru Anbalagane [2.6.32-100.23.32.el5]
- fixes ocfs2_page_mkwrite path (WenGang Wang) [orabug 10127979]
- Increase max guest memory from 32GB to 1TB (Guru Anbalagane) [orabug 11069098]

* Wed Jan 12 2011 Joe Jin [2.6.32-100.23.31.el5]
- [xen]: fix xenfb suspend/resume race (Joe Jin) [orabug 10389175]
- [net] exported firmware that missed in drivers (Zhenzhong Duan) [orabug 10433896]

* Wed Dec 29 2010 Guru Anbalagane [2.6.32-100.23.30.el5]
- minor fix to netconsole/bonding patch 
- add obsoletes/provides kernel-headers for package headers in specfile
  (Keshav Sharma)

* Thu Dec 23 2010 Guru Anbalagane [2.6.32-100.23.29.el5]
- update of following drivers from el6 (Joe Jin)
- [net] add netdev_print helper
- [net] update driver bnx2 to 2.0.8-j15
- [net] update driver bnx2x to 1.52.1-7
- [net] update driver cnic to 2.1.2
- [scsi] update driver bnx2i to 2.1.1
- [net] update driver tg3 to 3.108
- [net] update driver e100 to 3.5.24-k2-NAPI
- [net] update driver e1000 to 7.3.21-k6-NAPI
- [net] update driver e1000e to 1.2.7-k2
- [net] update driver igb to 2.1.0-k2
- [net] update driver igbvf to 1.0.0-k0
- [net] update driver ixgb to 1.0.135-k2-NAPI
- [net] update driver ixgbe to 2.0.62-k2
- [net] update driver ixgbevf to 1.0.0-k0
- [net] update driver be2net to 2.102.426r
- [scsi] update driver be2iscsi to 2.0.527.0
- [block] update driver cciss to 3.6.26
- update qla2xxx, qla4xxx, qlge, netxen_nic, qlcnic 
- [paravirt]ignore lazy mmu state change in interrupt path (Chuck Anderson) 
   [orabug 10285722]
- fix filp_close() race (Joe Jin) [orabug 10335998]
- [net] rework the patch for network drivers entropy.

* Tue Dec 21 2010 Joe Jin [2.6.32-100.23.28.el5]
- [XEN] ensure that all event channels start off bound to VCPU 0 [orabug 10389175]
- [NET] tehuti: fix incorrect firmware name.
#- [NET] fix pass incorrect parameter to ethtool_op_set_flags(). [orabug 10420948]
#- [BLOCK] revert cciss driver to 2.6.36.2

* Thu Dec 16 2010 Guru Anbalagane [2.6.32-100.23.27.el5]
- [SCSI]  wake xmit thread when killing session (Mike Christie) 

* Wed Dec 15 2010 Joe Jin [2.6.32-100.23.26.el5]
- enable hpilo support and update to upstream.
- update bnx2i to upstream.
- update 3w-xxxx to upstream.
- update 3w-9xxx to upstream.
- update be2iscsi to upstream.
- update aic94xx to upstream.
- update aic7xxx to upstream.
- update aic7xxx_old to upstream.
- update aacraid to upstream.
- update mlx4_ib to upstream.
- update fnic to upstream.
- update lpfc to upstream.
- update megaraid to vendor's drop.
- update cxgb3i to upstream.
- update mptsas to upstream.
- update mpt2sas to upstream
- update cciss to upstream.
- update cpqarray to upstream.

* Tue Dec 14 2010 Tina Yang <tina.yang@oracle.com>
- add netfront netconsole support (Tina Yang) [orabug 10115460]
- add netconsole over bonding support (Tina Yang) [orabug 10090882]

* Tue Dec 14 2010 Joe Jin [2.6.32-100.23.25.el5]
- Fix bnx2x driver panic. (Joe Jin) [orabug 10393605]
- Fix pv guest migrate hung (Joe Jin) [orabug 10114101]

* Wed Dec  8 2010 Guru Anbalagane [2.6.32-100.23.24.el5]
- update bnx2 to upstream(2.0.18) (Joe Jin) 
- update bnx2x to upstream(1.60.00-4)
- update cnic to upstream(2.2.6)
- update tg3 to upstream(3.115)
- update enic to upstream(1.4.1.6)
- update cxgb3 to upstream(1.1.4-ko)
- update e100 to upstream(3.5.24-k2-NAPI)
- update e1000 to upstream(7.3.21-k8-NAPI)
- update e1000e to upstream(1.2.7-k2)
- update igb to upstream(2.1.0-k2)
- update ixgb to upstream(1.0.135-k2-NAPI)
- update ixgbe to upstream(2.0.84-k2)
- update igbvf to upstream(1.0.0-k0)
- update ixgbevf to upstream(1.0.0-k0)
- update bna to upstream (2.3.2)
- update bfa to upstream(2.3.2.0)
- update be2net to upstream(2.103.175u)

* Mon Dec 07 2010 Keshav Sharma <keshav.sharma@oracle.com> [2.6.32-100.23.23.el5]
- Fixed debuginfo package build issue

* Mon Dec 06 2010 Keshav Sharma <keshav.sharma@oracle.com> [2.6.32-100.23.22.el5]
- Changes Package name from kernel to kernel-uek.

* Fri Dec  3 2010 Guru Anbalagane [2.6.32-100.23.21.el5]
- CVE's  from RHSA-2010:0842-01 

* Thu Dec  2 2010 Guru Anbalagane [2.6.32-100.23.20.el5]
- Back-port xen blkfront performance improvements from upstream [orabug 10115222]
- fix underflow of process execution time [ orabug 10132286]
- Find an unbound irq number in reverse order (high to low) [ orabug 10096783] 
- retry page fault when blocking on disk transfer [ orabug 10090014]

* Tue Nov 09 2010 Chris Mason [2.6.32-100.23.1.el5]
- Fix LACP bonding mode (Tina Yang)

* Mon Oct 25 2010 Chris Mason [2.6.32-100.22.1.el5]
- Fix grat arps on bonded interfaces (Tina Yang)

* Fri Oct 22 2010 Greg Marsden [2.6.32-100.21.1.el5]
* [rds] fix access issue with rds (Chris Mason) {CVE-2010-3904}

* Wed Oct 20 2010 Srinivas Eeda [2.6.32-100.20.1.el5_errata1]
- [fuse] linux-2.6.32-fuse-return-EGAIN-if-not-connected-bug-10154489.patch
- [net] linux-2.6.32-net-sched-fix-kernel-leak-in-act_police.patch
- [aio] linux-2.6.32-aio-check-for-multiplication-overflow-in-do_io_subm.patch

* Thu Sep 30 2010 Guru Anbalagane [2.6.32-100.20.1.el5]
- [fs] xfs: return inode fork offset in bulkstat for fsr (Dave Chinner)
- [fs] xfs: always use iget in bulkstat (Dave Chinner) {CVE-2010-2943}
- [fs] xfs: validate untrusted inode numbers during lookup (Dave Chinner) {CVE-2010-2943}
- [fs] xfs: rename XFS_IGET_BULKSTAT to XFS_IGET_UNTRUSTED (Dave Chinner) {CVE-2010-2943}
- [net] net sched: fix some kernel memory leaks (Eric Dumazet) {CVE-2010-2942}
- [fs] ocfs2: Don't walk off the end of fast symlinks (Joel Becker)

* Fri Sep 17 2010 Chris Mason [2.6.32-100.0.19.el5]
- Add fix for 32 bit compat bugs

* Thu Sep 16 2010 Chris Mason [2.6.32-100.0.18.el5]
- Fix scheduler regression with taskset
- Add support for new ixgbe card
- Fix kernel-doc build

* Tue Sep 14 2010 Chris Mason [2.6.32-100.0.17.el5]
- Add driver updates
- Update ocfs2
- Enable debug builds
- Enable doc builds
- Fix scsi rq completion on errors
- Add LPFC update for DIF/DIX

* Mon Sep 13 2010 Chris Mason [2.6.32-100.0.16.el5]
- Backport accept_local sys tunable, default to 1

* Thu Sep  9 2010 Chris Mason [2.6.32-100.0.15.el5]
- Panic on oops by default

* Wed Sep  8 2010 Chris Mason [2.6.32-100.0.14.el5]
- Stable update to 2.6.32.21
- Enable CONFIG_NO_HZ

* Mon Sep  6 2010 Chris Mason [2.6.32-100.0.13.el5]
- Fix RT scheduler bottleneck

* Fri Sep  3 2010 Guru Anbalagane [2.6.32-100.0.12.el5]
- fix ipmi sunoem cli busy issue

* Thu Sep 2 2010 Chri Mason [2.6.32-100.0.11.el5]
- Change network drivers to contribute to entropy
- Update OCFS2

* Fri Aug 27 2010 Chri Mason [2.6.32-100.0.10.el5]
- Add backported IPMI fixes from 2.6.35

* Wed Aug 25 2010 Chri Mason [2.6.32-100.0.9.el5]
- Use deadline io scheduler by default.

* Fri Aug 20 2010 Chri Mason [2.6.32-100.0.8.el5]
- Fix AIO spinlock contention w/igrab
- Add back ipmi_si tunables from 2.6.18 EL5

* Wed Aug 11 2010 Chuck Anderson [2.6.32-100.0.7.el5]
- add mechanism to change termal event report interval (chuck.anderson@oracle.com) [orabug 9966307]

* Tue Aug 03 2010 Chris Mason [2.6.32-100.0.6.el5]
- Turn on sched_compat_yield by default (java go faster please)
- Backport anon_vma fixes from 2.6.35

- Turn off AER error reporting for PCIe, it is causing problems on
  some Sun hardware.

* Mon Aug 02 2010 Srinivas Maturi <srinivas.maturi@oracle.com> [2.6.32-100.0.4.el5]
- Add pre install script to check for /dev/hd references in /etc/fstab

* Thu Jul 29 2010 Guru Anbalagane <guru.anbalagane@oracle.com> [2.6.32-100.0.3.el5]
- update mptsas to 4.22
- update megaraid_sas to 4.31-ieee
- backout changes related to nr_cpus > 1024

* Fri Jul 23 2010 Sunil Mushran <sunil.mushran@oracle.com> [2.6.32-100.0.2.el5]
- Updated ocfs2 version to 1.6.2
- Added datavolume mount option

* Thu Jul 22 2010 Chris Mason <chris.mason@oracle.com> [2.6.32-100.0.1.el5]
- Turn off the intel iommu by default for performance reasons

* Wed Jul 21 2010 Chris Mason <chris.mason@oracle.com> [2.6.32-100.el5]
- Change firewalling conntrack into a module
- Update the spec to include oracleasm provides:

* Tue Jul 20 2010 Chris Mason <chris.mason@oracle.com>
- Add module signing code

* Tue Jul 20 2010 Chris Mason <chris.mason@oracle.com>
- Add oracleasm 2.0.6

* Mon Jul 12 2010 Chris Mason <chris.mason@oracle.com>

- Update to 2.6.32.y kernel base
- Include patches to reduce IPC lock contention
- Backport receive packet steering code
- Expose list of msi-irqs per device into sysfs

